﻿;||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||;			
;|																															|;	
;| Title:				UB2D - 007_Lights																|;
;| Copyright: 	UnionBytes																			|;
;|																															|;
;| Date:				07.11.2018																			|;
;|																															|;
;||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||;


;- Engine
XIncludeFile "Includes\UB2D.pbi" 
If Not InitUnionBytes2DEngine(#UB2D_ENGINE_DEBUG)
	Debug "UB2D initialization failed!"
	End
EndIf

UB2D_OpenScreen(1280, 720, 32, "Union Bytes 2D Engine - Lights", #PB_Window_ScreenCentered|#PB_Window_SystemMenu, #False)


;- Variables
Global X.i, Y.i


;- Textures
; Loading the texture for our objects.
UB2D_LoadTexture(0, "Data\Textures\UB2D_B.png")	; Base color (albedo map)
UB2D_LoadTexture(1, "Data\Textures\UB2D_N.png") ; Normal map
UB2D_LoadTexture(2, "Data\Textures\UB2D_REM.png") ; REM map (Red = Roughness, Green = Emission, Blue = Metallic)

UB2D_LoadTexture(3, "Data\Textures\Floor_B.jpg"); Base color (albedo map)
UB2D_LoadTexture(4, "Data\Textures\Floor_N.jpg") ; Normal map
UB2D_LoadTexture(5, "Data\Textures\Floor_REM.jpg") ; REM map (Red = Roughness, Green = Emission, Blue = Metallic)


;- Materials
; Creating of a material for the UB2D Logo, with the default object PBR shader!
UB2D_CreateMaterial(0, UB2D_DefaultShaderID(#UB2D_SHADER_OBJ_PBR), 0)
UB2D_SetMaterialTexture(0, 1, 1) ; Set the normal map to index 1
UB2D_SetMaterialTexture(0, 2, 2) ; Set the rme map to index 2
UB2D_SetMaterialRoughness(0, 1.0) ; Set the roughness map multiplier
UB2D_SetMaterialMetallic(0, 1.0) ; Set the metallic map multiplier
UB2D_SetMaterialEmission(0, 0.0) ; Set the emission map multiplier
UB2D_SetMaterialSpecular(0, 0.8) ; Scale the specular amount

; Floor material, with the default object PBR shader!
UB2D_CreateMaterial(1, UB2D_DefaultShaderID(#UB2D_SHADER_OBJ_PBR), 3)
UB2D_SetMaterialTexture(1, 1, 4) ; Set the normal map to index 1
UB2D_SetMaterialTexture(1, 2, 5) ; Set the rme map to index 2
UB2D_SetMaterialRoughness(1, 1.0) ; Set the roughness map multiplier
UB2D_SetMaterialMetallic(1, 1.0) ; Set the metallic map multiplier
UB2D_SetMaterialEmission(1, 0.0) ; Set the emission map multiplier
UB2D_SetMaterialSpecular(1, 0.8) ; Scale the specular amount


;- Objects
; Creating the objects, with the material 0
UB2D_CreateObject(0, 0, 0.5, 0.5)
; Add a objects instance to the object 0, for objects and meshes is the position (0.0, 0.0)
; in the middle of the screen.
UB2D_AddObjectInstance(0, 0, 0.0, 0.0, 24.0)

; Creating the floor objects
UB2D_CreateObject(1, 1, 0.5, 0.5)
; Resizing of the original object
UB2D_ResizeObject(1, 320, 320)
; Add a objects instance for the floor
For Y = 0 To 2
	For X = 0 To 3
		UB2D_AddObjectInstance(1+(Y*5+X), 1, (X-1.5)*320.0, (Y-1)*320.0, 0.0)
	Next
Next


;- World light settings
; Reducing of the direct light and the ambientlight to see the point lights effects better
UB2D_SetDirectLightIntensity(0.00)
UB2D_SetAmbientLightColor($FFFFFFFF)

; Volumetric light is turned on by default
; UB2D_SetVolumetricLightState(#False)
; Increase the density of the folumetric effect
UB2D_SetVolumetricLightDensity(0.25)


;- Lights
; Create the  first Light on 200.0, -100.0 and a hight from 400.0 in world space!
UB2D_CreateLight(0, 400.0, 0.0, 128.0, 1024.0, $FF6AD5E5, #UB2D_LIGHT_SPOT)
; Set the light 0 cone angle to 45°
UB2D_SetLightConeAngle(0, 45.0)
; Set the light 0 intensity to 1.25
UB2D_SetLightIntensity(0, 1.25)

; Creation of the second light
UB2D_CreateLight(1, -100.0, -200.0, 64.0, 512.0, $FF1831D8, #UB2D_LIGHT_SPOT)
; Set the light 1 cone angle to 60°
UB2D_SetLightConeAngle(1, 80.0)
; Set the light 1 intensity to 1.25
UB2D_SetLightIntensity(1, 1.25)
; Set the light 1 direction to the left
UB2D_SetLightDirection(1, -0.5, 0.0, -0.5)
	
; Creation of the third light
UB2D_CreateLight(2, -50.0, 300.0, 200.0, 400.0, $FF1CDA48)


;- Main Loop
Repeat
	
	; Normal pure basic window event handling.
	Repeat
		Select UB2D_WindowEvent()
			Case #PB_Event_CloseWindow
				Break 2
			Case #PB_Event_None
				Break
		EndSelect 
	ForEver
	
	; Update of the control functions.
	UB2D_ExamineMouse()
	UB2D_ExamineKeyboard()
	
	; Set the first spot direction to the mouse position
	UB2D_SetLightDirection(0, UB2D_MouseX()-UB2D_LightPositionX(0)-UB2D_ScreenWidth()/2, UB2D_MouseY()-UB2D_LightPositionY(0)-UB2D_ScreenHeight()/2, -200)
	
	; Rotate the second spot light
	UB2D_RotateLight(1, 60.0*UB2D_TimeFactor())

	; Rotate the mesh instance
	UB2D_RotateObjectInstance(0, 15.0*UB2D_TimeFactor())
	
	; We have to replace now the funktion UB2D_BindFrameBuffer(#UB2D_FRAMEBUFFER_FINAL, #True, #True)
	; With the following one. After the scene is rendered, the final framebuffer will be set automatically
	UB2D_RenderScene()
	
	; Flip the buffers and draw it on the window.
	UB2D_FlipBuffers()
	
	; F12 makes a screenshot
	If UB2D_KeyboardReleased(#UB2D_KEY_F12)
		UB2D_ScreenShot("007_Lights.jpg")
	EndIf
	
ForEver

; Free the engine and all its resources.
UB2D_Free()
End

; IDE Options = PureBasic 5.62 (Windows - x64)
; EnableXP
; CompileSourceDirectory
; EnableCompileCount = 939
; EnableBuildCount = 0