﻿;||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||;			
;|																															|;	
;| Title:				UB2D - 011_Meshes																|;
;| Copyright: 	UnionBytes																			|;
;|																															|;
;| Date:				07.11.2018																			|;
;|																															|;
;||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||;


;- Engine
XIncludeFile "Includes\UB2D.pbi" 
If Not InitUnionBytes2DEngine(#UB2D_ENGINE_DEBUG)
	Debug "UB2D initialization failed!"
	End
EndIf

UB2D_OpenScreen(1280, 720, 32, "Union Bytes 2D Engine - Meshes", #PB_Window_ScreenCentered|#PB_Window_SystemMenu, #False)


; Adjustment of the sceen settings (not screen!)
UB2D_SetSceneGamma(0.6)


;- World light settings
; Loading and changing of the environmental sphere texture
UB2D_LoadEnvironmentTexture(0, "Data\Environments\DarkRoom.png")
UB2D_SetEnvironmentMap(0)

UB2D_SetDirectLightIntensity(0.2)
UB2D_SetAmbientLightColor($FF7E6562)

UB2D_SetVolumetricLightDensity(0.15)


;- Variables
Global *MeshInstance_Pumpkin.UB2D_MESH_INSTANCE
Global *MeshInstance_Vase.UB2D_MESH_INSTANCE

;- Fonts
UB2D_LoadFont(0, "Data\Fonts\Arial_20.tf") 


;- Textures
UB2D_LoadTexture(1, "Data\Textures\Vase_B.jpg")
UB2D_LoadTexture(2, "Data\Textures\Vase_N.jpg")
UB2D_LoadTexture(3, "Data\Textures\Vase_REM.jpg")

UB2D_LoadTexture(4, "Data\Textures\Pumpkin_B.jpg")
UB2D_LoadTexture(5, "Data\Textures\Pumpkin_N.jpg")
UB2D_LoadTexture(6, "Data\Textures\Pumpkin_REM.jpg")

UB2D_LoadTexture(7, "Data\Sprites\Blender OBJ-Export.jpg")
UB2D_LoadTexture(8, "Data\Sprites\Blender DAE-Export.jpg")

;- Materials
; Important, use the right material shader! #UB2D_SHADER_3D_PBR!
UB2D_CreateMaterial(0, UB2D_DefaultShaderID(#UB2D_SHADER_3D_PBR), 1)
UB2D_SetMaterialTexture(0, 1, 2)
UB2D_SetMaterialTexture(0, 2, 3)

UB2D_CreateMaterial(1, UB2D_DefaultShaderID(#UB2D_SHADER_3D_PBR), 4)
UB2D_SetMaterialTexture(1, 1, 5)
UB2D_SetMaterialTexture(1, 2, 6)

UB2D_CreateMaterial(2, UB2D_DefaultShaderID(#UB2D_SHADER_SPR_DEFAULT), 7)
UB2D_CreateMaterial(3, UB2D_DefaultShaderID(#UB2D_SHADER_SPR_DEFAULT), 8)

; Meshes
; Load a OBJ and DAE mesh. The mesh has to be triangulated exported. Also it needs UV and normals!
; Because we have a ortograpic view in the screen space, 1 unit is 1 Pixel, so we have to
; scale it up! For this model we use a scale of 400.
UB2D_LoadDAEMesh(0, "Data\Meshes\Vase.dae", 0, 100.0)

UB2D_LoadOBJMesh(1, "Data\Meshes\Pumpkin.obj", 1, 250.0)

; UB2D_LoadOBJMesh(1, "Data\Meshes\Pumpkin.obj", -1, 400.0)
; For a better use and for a easier heandling of the instance we use in this case a pointer.
*MeshInstance_Vase = UB2D_AddMeshInstance(#UB2D_ANY, 0, -50.0, 100.0, 0.0)	
; Rotate the mesh instance to the side.
UB2D_RotateMeshInstance(*MeshInstance_Vase, 60.0, 0.0, 0.0)

*MeshInstance_Pumpkin = UB2D_AddMeshInstance(#UB2D_ANY, 1, 140.0, -160.0, 0.0)	
UB2D_RotateMeshInstance(*MeshInstance_Pumpkin, 0.0, 0.0, 0.0)
UB2D_Attach(*MeshInstance_Vase, *MeshInstance_Pumpkin)


;- Sprites
UB2D_CreateSprite(0, 2, 0.0, 1.0)
UB2D_CreateSprite(1, 3, 1.0, 1.0)


;- Lights
UB2D_CreateLight(1, 400, -200, 1000.0, 5000.0, $FFFFFFFF)
UB2D_SetLightIntensity(1, 1.0)
UB2D_CreateLight(2, -600, 600, 250.0, 3000.0, $FF0000FF)
UB2D_SetLightIntensity(2, 1.2)


;- Main Loop
Repeat
	
	Repeat
		Select UB2D_WindowEvent()
			Case #PB_Event_CloseWindow
				Break 2
			Case #PB_Event_None
				Break
		EndSelect 
	ForEver
	
	; UB2D GUI event handling.
	Select UB2D_Event()
		Case #UB2D_EVENT_GADGET
			Select UB2D_EventGadget()
			
			EndSelect
	EndSelect
	
	; Mouse and keyboard functions.
	UB2D_ExamineMouse()
	UB2D_ExamineKeyboard()
	
	; F1 open the engine control window.
	If UB2D_KeyboardReleased(#UB2D_KEY_F1)
		If UB2D_EngineControlStatus()
			UB2D_CloseEngineControl()
		Else
			UB2D_OpenEngineControl(0, 0)
		EndIf
	EndIf
	
	; Right mouse butten + move in x and y direction, let us rotate the mesh.
	If UB2D_MouseButtonPushed(#UB2D_MOUSEBUTTON_RIGHT)
		UB2D_RotateMeshInstance(*MeshInstance_Vase, -UB2D_MouseDeltaY()*0.5, UB2D_MouseDeltaX()*0.5, 0.0)
 	EndIf
 	
 	; Render the scene
 	UB2D_RenderScene()
 	
 	; Display info text on the screen.
 	UB2D_DisplayText(0, 20, 20, "F1: Open Engine Control!", $FFFFFFFF) 	
 	UB2D_DisplayText(0, 20, 40, "Right mouse butten (RMB) + move mouse in x and y direction: Rotate mesh!", $FFFFFFFF)
 	
 	; Display sprites and text
 	UB2D_DisplaySprite(0, 0, UB2D_ScreenHeight())
 	UB2D_DisplaySprite(1, UB2D_ScreenWidth(), UB2D_ScreenHeight())
 	
 	UB2D_DisplayText(#UB2D_DEFAULT, 256, UB2D_ScreenHeight()-32, "On the sides you see the Blender exporter settings!")
 	
 	; Display GUI, like the engine control window.
	UB2D_DisplayGUI()
	
	; Flip the buffers and draw it on the window.
	UB2D_FlipBuffers()
	
	; F12 makes a screenshot
	If UB2D_KeyboardReleased(#UB2D_KEY_F12)
		UB2D_ScreenShot("011_Meshes.jpg")
	EndIf
	
ForEver

; Free the engine and all its resources.
UB2D_Free()
End

; IDE Options = PureBasic 5.62 (Windows - x64)
; EnableXP
; CompileSourceDirectory
; EnableCompileCount = 833
; EnableBuildCount = 0